// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj06_db.h was generated from the DDL file obj06_db.ddl.
#include "obj06_db.h"

// obj06_db_api.h was generated from the DDL file obj06_db.ddl. with 
// the -cpp option
#include "obj06_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj06 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) Several new artists are added to the database
//   3) For each artist a couple albums are added to the database
//   4) For each album a tracks are added to the database
//   5) A list of each artist, their albums, and the album's tracks is 
//      displayed

// The DDL (Database Definition Language) is in obj06_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Db->KeyFindBy
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor->GetMembers
//      Cursor->Connect
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj06_db' in this
// project directory.

extern const char *the_doors_tracks[2];
extern const char *strange_days_tracks[3];
extern const char *waiting_tracks[2];
extern const char *the_rolling_stones_tracks[2];
extern const char *out_of_our_heads_tracks[2];
extern const char *beggars_banquet_tracks[2];
extern const char *tattoo_you_tracks[3];
extern const char *bleach_tracks[2];
extern const char *nevermind_tracks[2];
extern const char *in_utero_tracks[3];

class obj06
{
public:
    Db_obj06_db   db;
    Cursor_album  album_cursor;
    Cursor_artist artist_cursor;
    Cursor_track  track_cursor;

    void InsertData () throw (rdm_exception&);
    void DisplayCollection () throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
    void AddAlbum (      char *artist_name,
                   const char *album_title,
                   uint16_t    num_tracks,
                   const char *track_names[]) throw (rdm_exception&);
};

void obj06::AddAlbum(
    char     *artist_name,
    const char     *album_title,
    uint16_t        num_tracks,
    const char     *track_names[])
{
    int32_t ii;
    struct album album_rec;
    struct track track_rec;

    strcpy(album_rec.title, album_title);

    try
    {
        // Make artist cursor a "key" cursor
        artist_cursor = db.Get_artist_recordsBy_name();
        
        // Do a key search for the requested artist
        artist_cursor.KeyFindBy_name(artist_name);
        if(!artist_cursor.AtRecord())
        {
            cout << "Unable to find artist with the name " << artist_name << endl;
            return;
        }

        // Add the album record
        album_cursor = db.New_album_recordWithFieldValues(&album_rec);

        // Connect it to the artist we found
        album_cursor.Connect_artist_album(artist_cursor);

        // Add the tracks for the album
        for (ii = 0; ii < num_tracks; ii++)
        {
            strcpy(track_rec.name, track_names[ii]);

            // Add the track record
            track_cursor = db.New_track_recordWithFieldValues(&track_rec);

            // Associate the track with the album
            track_cursor.Connect_album_track(album_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error adding an album (" << e.error() << " - " << e.what() << ")" << endl;
        throw (e);
        return;
    }
    return;
}

void obj06::Initialize()
{
    // Open the database and initialize it
    try
    {
        db = db.Open("obj06_db", "x");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj06_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj06::InsertData()
{
    struct artist artist_rec;

    // Create the artists, albums, and tracks in the database
    try
    {
        // Add the artists
        strcpy(artist_rec.name, "The Doors");
        db.New_artist_recordWithFieldValues(&artist_rec);

        strcpy(artist_rec.name, "The Rolling Stones");
        db.New_artist_recordWithFieldValues(&artist_rec);

        strcpy(artist_rec.name, "Nirvana");
        db.New_artist_recordWithFieldValues(&artist_rec);
        
        // Add the albums
        AddAlbum("The Doors","The Doors", RLEN(the_doors_tracks), the_doors_tracks);
        AddAlbum("The Doors","Strange Days", RLEN(strange_days_tracks), strange_days_tracks);
        AddAlbum("The Doors","Waiting for the Sun", RLEN(waiting_tracks), waiting_tracks);
        AddAlbum("The Rolling Stones","The Rolling Stones", RLEN(the_rolling_stones_tracks), the_rolling_stones_tracks);
        AddAlbum("The Rolling Stones","Out of our Heads", RLEN(out_of_our_heads_tracks), out_of_our_heads_tracks);
        AddAlbum("The Rolling Stones","Beggars Banquest", RLEN(beggars_banquet_tracks), beggars_banquet_tracks);
        AddAlbum("The Rolling Stones","Tattoo You", RLEN(tattoo_you_tracks), tattoo_you_tracks);
        AddAlbum("Nirvana", "Bleach", RLEN(bleach_tracks), bleach_tracks);
        AddAlbum("Nirvana", "Nevermind", RLEN(nevermind_tracks), nevermind_tracks);
        AddAlbum("Nirvana", "In Utero", RLEN(in_utero_tracks), in_utero_tracks);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj06::DisplayCollection()
{
    struct artist artist_rec;
    struct album  album_rec;
    struct track  track_rec;
    try
    {
        // Loop through each artist record and display the albums they are 
        // associated with
        artist_cursor = db.Get_artist_recordsBy_name();
        while (!artist_cursor.AfterLast())
        {
            artist_cursor.GetFieldValues(&artist_rec);
            cout << endl << artist_rec.name << endl;

            // Get the the albums for this artist
            album_cursor = artist_cursor.Get_artist_album_members();
            while (!album_cursor.AfterLast())
            {
                album_cursor.GetFieldValues(&album_rec);
                cout << '\t' << album_rec.title << endl;

                // Get the tracks for this album
                track_cursor = album_cursor.Get_album_track_members();
                while ( !track_cursor.AfterLast() )
                {
                    track_cursor.GetFieldValues(&track_rec);
                    cout << "\t\t" << track_rec.name << endl;

                    track_cursor++;
                }
                album_cursor++;
            }
            artist_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj06::Cleanup()
{
    return;
}

int32_t main(void)
{
    obj06 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();
        obj.DisplayCollection();
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}

const char *the_doors_tracks[2] = 
{
    "Break on Through (To the Other Side)",
    "Soul Kitchen" ,
};

const char *strange_days_tracks[3] = 
{
    "Strange Days",
    "You're Lost Little Girl",
    "Love Me Two Times",
};

const char *waiting_tracks[2] = 
{
    "Hello, I Love You",
    "Love Street",
};

const char *the_rolling_stones_tracks[2] =
{
    "Not Fade Away",
    "Route 66",
};

const char *out_of_our_heads_tracks[2] =
{
    "She Said Yeah",
    "Mercy, Mercy",
};

const char *beggars_banquet_tracks[2] =
{
    "Sympathy for the Devil",
    "No Expectations",
};

const char *tattoo_you_tracks[3] =
{
    "Start Me Up",
    "Hang Fire",
    "Slave",
};

const char * bleach_tracks[2] =
{
    "Blew",
    "Floyd The Barber",
};

const char * nevermind_tracks[2] =
{
    "Smells Like Teen Spirit",
    "In Bloom",
};

const char * in_utero_tracks[3] =
{
    "Serve the Servants",
    "Scentless Apprentice",
    "Heart-Shaped Box",
};
